# ================================================================
# LineSegment.py: Line segments are defined by end points (x1, y1) 
# and (x2, y2). Compute length and angle of the line segment in
# radians. 
# 
# Written by: Mark Austin                            October, 2022 
# ================================================================

import math

from Point import Point

class LineSegment:
  __length = 0
  __angle  = 0

  def __init__(self, x1, y1, x2, y2 ):
    self.__pt1 = Point(x1,y1) 
    self.__pt2 = Point(x2,y2) 
    self.__length = self.__pt1.distance(self.__pt2)
    self.__angle  = self.getAngle()

  # Compute angle (radians) for coordinates in four quadrants ....

  def getAngle(self):
     dX = self.__pt2.get_xCoord() - self.__pt1.get_xCoord();
     dY = self.__pt2.get_yCoord() - self.__pt1.get_yCoord();

     if dY >  0.0 and dX == 0.0:
        angle = math.pi/2.0
     if dY >= 0.0 and dX > 0.0:
        angle = math.atan( dY/dX )
     if dY >= 0.0 and dX < 0.0:
        angle = math.pi + math.atan( dY/dX )
     if dY < 0.0 and dX < 0.0:
        angle = math.pi + math.atan( dY/dX )
     if dY < 0.0 and dX >= 0.0:
        angle = 2*math.pi + math.atan( dY/dX )

     return angle

  # String represention of line segment ...

  def __str__(self):
     x1 = self.__pt1.get_xCoord();
     y1 = self.__pt1.get_yCoord();
     x2 = self.__pt2.get_xCoord();
     y2 = self.__pt2.get_yCoord();
     return "--- LineSegment: (x1,y1) = (%5.2f, %5.2f), (x2,y2) = (%5.2f, %5.2f), angle = %.2f, length = %.2f" % (
                 x1, y1, x2, y2, self.__angle, self.__length )

