# ====================================================================
# Stack.py: Linked list implementation of a stack. Key points:
# 
# --- Stacks follow the last-in-first-out rule, where the last element
#     that is inserted, is the first element that comes out.
# --- Insertion and deletion of elements in a stack take place from
#     one end only.
# 
# Main operations (and time complexities):
# 
# --- push (Insert) - O(1)
# ---  pop (Remove) - O(1)
# --- peek (Retrieve the top element) - O(1)
# 
# ====================================================================

# Generic stack node ...

class Node():
    def __init__(self, data):
        self.data = data
        self.next = None

# Stack class ...

class Stack():
    def __init__(self):
        self.top = None
        self.bottom = None
        self.length = 0

    # Peek at the top element ...

    def peek(self):
        if self.top is None:
            return None
        return self.top.data

    # Push operation (insert an element at the top of the stack) ...
   
    def push(self, data):
        new_node = Node(data)
        if self.top == None:
            self.top = new_node
            self.bottom = new_node
        else: 
            new_node.next = self.top
            self.top = new_node
        self.length += 1

    # Pop operation (remove an element at the top of the stack) ...

    def pop(self):
        if self.top == None: #If the stack is empty, we print an appropriate message
            print("--- Stack empty")
        else: 
            self.top = self.top.next
            self.length -= 1
            if(self.length == 0): 
                self.bottom = None

    # Print stack ...

    def print_stack(self):
        if self.top == None:
            print("--- Empty")
        else:
            current_pointer = self.top
            i = self.length
            while(current_pointer!=None):
                print("--- Item %d: %s ..." % (i, current_pointer.data))
                current_pointer = current_pointer.next
                i = i - 1

