# ======================================================================
# TestMatrixEigenvalues.py: Compute matrix eigenvalues and eigenvectors.
# 
# ======================================================================

from numpy import array
from numpy.linalg import eig

from scipy.linalg import lu
from scipy.linalg import qr

# ===============================================================
# Function to print two-dimensional matrices ...
# ===============================================================

def PrintMatrix(a):
    for row in a:
        for col in row:
            print("{:8.2f}".format(col), end=" ")
        print("")

# ===============================================================
# main method ...
# ===============================================================

def main():
    print("--- Enter TestMatrixEigenvalues.main()       ... ");
    print("--- ======================================== ... ");

    # Define square 3x3 matrix ...

    print("---");
    print("--- Part 01: Define square 3x3 matrix ...");
    print("---");

    A = array([ [1,2,3],
                [4,5,6],
                [7,8,9]
              ])
    PrintMatrix(A)
    print(A.shape)

    # Compute eigenvalues and eigenvectors ...

    print("---");
    print("--- Part 02: Compute eigenvalues and eigenvectors ...");
    print("---");

    evals, evects = eig(A)

    print(evals)
    print(evects)
    print(evals.shape)
    print(evects.shape)

    # Factorize A ...

    print("---");
    print("--- Part 03: Factorize A --> P, L, U ...");
    print("---");

    P, L, U = lu(A)

    print(P)
    print(L)
    print(U)
    print(A.shape)
    print(P.shape)
    print(L.shape)
    print(U.shape)

    # Reconstruct A from P.L.U

    print("---");
    print("--- Part 04: Reconstruct A from P.L.U ...");
    print("---");

    B = P.dot(L).dot(U)

    print(B)
    print(B.shape)

    # Define square matrix

    print("---");
    print("--- Part 05: Define rectangular matrix A ...");
    print("---");

    A = array([ [1,2], [4,5], [7,8] ])

    print(A)

    # Factorize

    print("---");
    print("--- Part 06: Compute QR decomposition of A ...");
    print("---");

    Q, R = qr(A, mode='full')

    print(Q)
    print(R)
    print(A.shape)
    print(Q.shape)
    print(R.shape)

    print("---");
    print("--- Part 07: Reconstruct A from Q.R ...");
    print("---");

    B = Q.dot(R)
    print(B)
    print(B.shape)

    print("--- ======================================== ... ");
    print("--- Leave TestMatrixEigenvalues.main()       ... ");

# call the main method ...

main()
