# ===============================================================
# TestMultiLayer-XOR-Gate02.py: Train and test boolean XOR gate
# with one hidden layer.
# 
# Required Behavior: 
# 
# ----------------------------------
#    Input 1     Input 2      Output 
# ----------------------------------
#          0           0           0 
#          0           1           1        
#          1           0           1        
#          1           1           0        
# ----------------------------------
# 
# Network Architecture:
# --------------------------------------------------------------- 
# 2 input nodes, 4 nodes on hidden layer, 1 node on output. 
# 
# Weighted edges: 
# 
# input  layer --> hidden layer: connected by 2x4 = 8 weights. 
# hidden layer --> output layer: connected by 4 weights. 
# 
# Written by: Mark Austin                           November 2020 
# ===============================================================

import numpy as np

# Sigmoid function ...

def sigmoid(x):
    return 1.0/(1+ np.exp(-x))

# Sigmoid derivative function ...

def sigmoid_derivative(x):
    return x * (1.0 - x)

# Neural Network model ...

class NeuralNetwork:
    def __init__(self, x, y):
        self.input      = x
        self.weights1   = np.random.rand(self.input.shape[1],4) 
        self.weights2   = np.random.rand(4,1)                 
        self.y          = y
        self.output     = np.zeros(self.y.shape)

    def feedforward(self):
        self.layer1 = sigmoid( np.dot( self.input, self.weights1))
        self.output = sigmoid( np.dot(self.layer1, self.weights2))

    def backprop(self):
        # Use chain rule to find derivative of the loss
        # function with respect to weights2 and weights1

        d_weights2 = np.dot(self.layer1.T, (2*(self.y - self.output) * sigmoid_derivative(self.output)))
        d_weights1 = np.dot(self.input.T,  (np.dot(2*(self.y - self.output) * sigmoid_derivative(self.output), self.weights2.T) * sigmoid_derivative(self.layer1)))

        # Update the weights with the derivative (slope) of the loss function

        self.weights1 += d_weights1
        self.weights2 += d_weights2

# main method ...

def main():
    print("--- Enter TestTensorFlow-XOR-Gate02.main()   ... ");
    print("--- ======================================== ... ");

    # Initialize training data  ...

    X = np.array([[0,0],
                  [0,1],
                  [1,0],
                  [1,1]])

    y = np.array([[0],[1],[1],[0]])

    # Create neural network model ...

    print("--- Crearte neural network model ... ");

    nn = NeuralNetwork(X,y)

    # Train network and apply back propogation ...

    print("--- Train network ... ");

    for i in range(1500):
        nn.feedforward()
        nn.backprop()

    # Verify and print results ...

    print("--- Network weights ... ");
    print("--- --------------------------------------------- ... ");
    print("--- Layer 1");
    print( nn.weights1 )
    print("--- Layer 2");
    print( nn.weights2 )

    print("--- Verify and print results ... ");
    print("--- --------------------------------------------- ... ");

    print( nn.output)

    print("--- ======================================== ... ");
    print("--- Finished!! ... ");

# call the main method ...

main()
