# ===============================================================
# TestMatrixEquations01.py: Compute solution to matrix equations.
# 
# Written by: Mark Austin                           November 2022 
# ===============================================================

import numpy as np 
from numpy.linalg import matrix_rank

# ===============================================================
# Function to print two-dimensional matrices ...
# ===============================================================

def PrintMatrix(name, a):
    print("Matrix: {:s} ".format(name) );
    for row in a:
        for col in row:
            print("{:8.3f}".format(col), end=" ")
        print("")

# ===============================================================
# main method ...
# ===============================================================

def main():
    print("--- Enter TestMatrixEquations01.main()      ... ");
    print("--- ======================================== ... ");

    print("--- ");
    print("--- Part 1: Create test matrices ... ");
    print("--- ");

    A = np.array( [ [ 3, -6,  7],
                    [ 9,  0, -5],
                    [ 5, -8,  6] ]);
    PrintMatrix("A", A);

    B = np.array([ [3], [3], [-4] ]);
    PrintMatrix("B", B);

    print("--- ");
    print("--- Part 2: Check properties of matrix A ... ");
    print("--- ");

    rank = matrix_rank(A)
    det  = np.linalg.det(A)

    print("--- Matrix A: rank = {:f}, det = {:f}  ...".format(rank, det) );

    print("--- ");
    print("--- Part 3: Solve A.x = B ... ");
    print("--- ");

    x = np.linalg.solve(A, B)
    PrintMatrix("x", x);

    print("--- ======================================== ... ");
    print("--- Leave TestMatrixEquations01.main()      ... ");

# call the main method ...

main()
