# ===============================================================
# TestMatrixLibrary.py: Matrix library in numpy ...
# 
# ===============================================================

import numpy as np  # Make numpy available using np.
import numpy.matlib

# ===============================================================
# Function to print two-dimensional matrices ...
# ===============================================================

def PrintMatrix(name, a):
    print("Matrix: {:s} ".format(name) );
    for row in a:
        for col in row:
            print("{:8.3f}".format(col), end=" ")
        print("")

# ===============================================================
# main method ...
# ===============================================================

def main():
    print("--- Enter TestNumpyMatrixLibrary.main()      ... ");
    print("--- ======================================== ... ");

    # Part 1: Create a basic two-dimensional array ...

    print("--- ");
    print("--- Part 01: Create basic two-dimensional array ... ");
    print("--- ");

    matrix01 = np.array([ (1, 2, 3), (4, 5, 6) ])
    PrintMatrix( "matrix01", matrix01 );

    # Part 2: Demonstrate numpy.matlib.empty() ...

    print("--- ");
    print("--- Part 02: Demonstrate np.zeros( shape = (4,4) ) ... ");
    print("--- ");

    matrix02 = np.zeros(shape=(4, 4))
    PrintMatrix("matrix02", matrix02 );

    # Part 3: Demonstrate numpy.matlib.eye() ...

    print("--- ");
    print("--- Part 03: Demonstrate numpy.matlib.eye()   ... ");
    print("--- ");

    matrix03 = np.eye(4, dtype = float)
    PrintMatrix("matrix03", matrix03 );

    # Part 4: Demonstrate numpy.matlib.ones() ...

    print("--- ");
    print("--- Part 04: Demonstrate numpy.ones()   ... ");
    print("--- ");

    matrix04 = np.ones((4,4));
    PrintMatrix("matrix04", matrix04 );

    # Part 5: Demonstrate numpy.matlib.ones() ...

    print("--- ");
    print("--- Part 05: Demonstrate numpy.random() ... ");
    print("--- ");

    matrix05 = np.random.random((4,4));
    PrintMatrix("matrix05", matrix05 );

    # Part 6: Demonstrate matrix shape manipulation ...

    print("--- ");
    print("--- Part 06: Demonstrate matrix shape manipulation ... ");
    print("--- ");

    print("--- Create 1-d array of 20 random numbers ...");

    matrix06 = np.random.random((20,1));
    PrintMatrix("matrix06", matrix06 );

    print("--- Reshape to (10x2) matrix ...");

    PrintMatrix ( "matrix06 (reshaped)", matrix06.reshape(10,2) )

    # Part 7: Properties of two-dimensional array ...

    print("--- ");
    print("--- Part 07: Properties of two dimensional array ... ");
    print("--- ");

    B = np.array([ (0,0), (0,1), (1,0), (1,1) ], "float32")
    PrintMatrix("B", B)

    print("--- matrix shape: ", B.shape );

    print("--- ======================================== ... ");
    print("--- Leave TestNumpyMatrixLibrary.main()      ... ");

# call the main method ...

main()
