# ========================================================================
# TestSeattleWeather01.py: Time series model of weather in Seattle.
# ========================================================================

from datetime import datetime
import pandas as pd
import matplotlib.pyplot as plt

# main method ...

def main():
    print("--- Enter TestSeattleWeather01.main()        ... ");
    print("--- ======================================== ... ");

    print("--- ");
    print("--- Part 1: Get seattle weather conditions ... ");
    print("---       : date,precipitation,temp_max,temp_min,wind,weather ... ");
    print("--- ");

    data = pd.read_csv('../data/seattle-weather.csv')
    df = pd.DataFrame(data, columns = ['date', 'temp_min', 'temp_max'])

    print(df)

    print("--- ");
    print("--- Part 2: Set date as the index ... ");
    print("---       : df['date'] = pd.to_datetime(df['date'], format='%Y-%m-%d') ... ");
    print("--- ");

    df['date'] = pd.to_datetime( df['date'], format='%Y-%m-%d')
    df.index = df['date']
    del df['date']

    print(df)

    print("--- ");
    print("--- Part 3: Create simple plot ... ");
    print("--- ");

    df.plot(figsize=(15, 6))
    plt.legend(["Min Temp", "Max Temp"], loc='upper left')
    plt.show()

    print("--- ");
    print("--- Part 4: Filter weather for 2012, 2013, 2014, 2015 ... ");
    print("--- ");

    df2012 = df[ df.index.year.isin([2012]) ]
    df2013 = df[ df.index.year.isin([2013]) ]
    df2014 = df[ df.index.year.isin([2014]) ]
    df2015 = df[ df.index.year.isin([2015]) ]

    print("--- ");
    print("--- Part 5: Time-history of max/min temperatures 2012 through 2015 ... ");
    print("--- ");

    ax = plt.gca()
    df2012.plot(kind='line',y='temp_min',color= 'blue',ax=ax)
    df2012.plot(kind='line',y='temp_max',color= 'blue',ax=ax)
    df2013.plot(kind='line',y='temp_min',color= 'red',ax=ax)
    df2013.plot(kind='line',y='temp_max',color= 'red',ax=ax)
    df2014.plot(kind='line',y='temp_min',color= 'orange',ax=ax)
    df2014.plot(kind='line',y='temp_max',color= 'orange',ax=ax)
    df2015.plot(kind='line',y='temp_min',color= 'green',ax=ax)
    df2015.plot(kind='line',y='temp_max',color= 'green',ax=ax)

    plt.ylabel('Max/min temperature (C)')
    plt.xlabel('Time (Jan 1, 2012 through Dec 31, 2015)')
    plt.title('Time-history of Max/Min Daily Temperature in Seattle')

    ax.legend(["2012: Min Temp", "2012: Max Temp", "2013: Min Temp", "2013: Max Temp",
               "2014: Min Temp", "2014: Max Temp", "2015: Min Temp", "2015: Max Temp"],
               loc='upper left')

    plt.grid()
    plt.show()

    print("--- ");
    print("--- Part 6: Map weather to day-of-the-year ... ");
    print("--- ");

    df2012.index = pd.DatetimeIndex( df2012.index ).dayofyear
    df2013.index = pd.DatetimeIndex( df2013.index ).dayofyear
    df2014.index = pd.DatetimeIndex( df2014.index ).dayofyear
    df2015.index = pd.DatetimeIndex( df2015.index ).dayofyear

    print("--- ");
    print("--- Part 7: Summary of transformed weather data ... ");
    print("--- ");

    print("--- Seattle weather for 2012 ");
    print (df2012)
    print("--- Seattle weather for 2013 ");
    print (df2013.head() )
    print("--- Seattle weather for 2014 ");
    print (df2014.head() )
    print("--- Seattle weather for 2015 ");
    print (df2015.head() )

    print("--- ");
    print("--- Part 8: Superimposed plot of weather measurements ... ");
    print("--- ");

    ax = plt.gca()
    df2012.plot(kind='line',y='temp_min',color= 'blue',ax=ax)
    df2012.plot(kind='line',y='temp_max',color= 'blue',ax=ax)
    df2013.plot(kind='line',y='temp_min',color= 'red',ax=ax)
    df2013.plot(kind='line',y='temp_max',color= 'red',ax=ax)
    df2014.plot(kind='line',y='temp_min',color= 'orange',ax=ax)
    df2014.plot(kind='line',y='temp_max',color= 'orange',ax=ax)
    df2015.plot(kind='line',y='temp_min',color= 'green',ax=ax)
    df2015.plot(kind='line',y='temp_max',color= 'green',ax=ax)

    plt.ylabel('Max/min temperature (C)')
    plt.xlabel('Day of the year (Jan. 1 through Dec. 31, 2012-2015)')
    plt.title('Superimposed Time-history of Max/Min Daily Temperature in Seattle')

    ax.legend(["2012: Min Temp", "2012: Max Temp", "2013: Min Temp", "2013: Max Temp",
               "2014: Min Temp", "2014: Max Temp", "2015: Min Temp", "2015: Max Temp"],
               loc='upper left')

    plt.grid()
    plt.show()

    print("--- ======================================== ... ");
    print("--- Leave TestSeattleWeather01.main()        ... ");

# call the main method ...

main()
